@@ -4,20 +4,20 @@
 
   Author: Pekka Riikonen <priikone@silcnet.org>
 
-  Copyright (C) 1997 - 2002 Pekka Riikonen
+  Copyright (C) 1997 - 2005 Pekka Riikonen
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
-  
+
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
 */
-/* $Id: idlist.c,v 1.66 2002/05/06 20:11:02 priikone Exp $ */
+/* $Id: idlist.c,v 1.79.2.11 2005/04/11 21:26:31 priikone Exp $ */
 
 #include "serverincludes.h"
 #include "idlist.h"
@@ -76,19 +76,27 @@ void silc_idlist_del_data(void *entry)
     silc_hash_free(idata->hash);
   if (idata->public_key)
     silc_pkcs_public_key_free(idata->public_key);
+
+  idata->send_key = NULL;
+  idata->receive_key = NULL;
+  idata->rekey = NULL;
+  idata->hmac_send = NULL;
+  idata->hmac_receive = NULL;
+  idata->hash = NULL;
+  idata->public_key = NULL;
 }
 
 /* Purges ID cache */
 
 SILC_TASK_CALLBACK_GLOBAL(silc_idlist_purge)
 {
+  SilcServer server = app_context;
   SilcIDListPurge i = (SilcIDListPurge)context;
 
-  SILC_LOG_DEBUG(("Start"));
+  SILC_LOG_DEBUG(("Purging cache"));
 
   silc_idcache_purge(i->cache);
-  silc_schedule_task_add(i->schedule, 0, 
-			 silc_idlist_purge,
+  silc_schedule_task_add(server->schedule, 0, silc_idlist_purge,
 			 (void *)i, i->timeout, 0,
 			 SILC_TASK_TIMEOUT, SILC_TASK_PRI_LOW);
 }
@@ -104,16 +112,25 @@ SILC_TASK_CALLBACK_GLOBAL(silc_idlist_purge)
    when new server connects to us. We also add ourselves to cache with
    this function. */
 
-SilcServerEntry 
-silc_idlist_add_server(SilcIDList id_list, 
+SilcServerEntry
+silc_idlist_add_server(SilcIDList id_list,
 		       char *server_name, int server_type,
 		       SilcServerID *id, SilcServerEntry router,
 		       void *connection)
 {
   SilcServerEntry server;
+  char *server_namec = NULL;
 
   SILC_LOG_DEBUG(("Adding new server entry"));
 
+  /* Normalize name.  This is cached, original is in server context.  */
+  if (server_name) {
+    server_namec = silc_identifier_check(server_name, strlen(server_name),
+					 SILC_STRING_UTF8, 256, NULL);
+    if (!server_namec)
+      return NULL;
+  }
+
   server = silc_calloc(1, sizeof(*server));
   server->server_name = server_name;
   server->server_type = server_type;
@@ -121,9 +138,10 @@ silc_idlist_add_server(SilcIDList id_list,
   server->router = router;
   server->connection = connection;
 
-  if (!silc_idcache_add(id_list->servers, server->server_name, 
+  if (!silc_idcache_add(id_list->servers, server_namec,
 			(void *)server->id, (void *)server, 0, NULL)) {
     silc_free(server);
+    silc_free(server_namec);
     return NULL;
   }
 
@@ -145,25 +163,25 @@ silc_idlist_find_server_by_id(SilcIDList id_list, SilcServerID *id,
   SILC_LOG_DEBUG(("Server ID (%s)",
 		  silc_id_render(id, SILC_ID_SERVER)));
 
-  if (!silc_idcache_find_by_id_one(id_list->servers, (void *)id, 
+  if (!silc_idcache_find_by_id_one(id_list->servers, (void *)id,
 				   &id_cache))
     return NULL;
 
   server = (SilcServerEntry)id_cache->context;
 
-  if (ret_entry)
-    *ret_entry = id_cache;
-
-  if (server && registered && 
+  if (server && registered &&
       !(server->data.status & SILC_IDLIST_STATUS_REGISTERED))
     return NULL;
 
+  if (ret_entry)
+    *ret_entry = id_cache;
+
   SILC_LOG_DEBUG(("Found"));
 
   return server;
 }
 
-/* Find server by name */
+/* Find server by name.  The 'name' must be normalized already. */
 
 SilcServerEntry
 silc_idlist_find_server_by_name(SilcIDList id_list, char *name,
@@ -178,14 +196,14 @@ silc_idlist_find_server_by_name(SilcIDList id_list, char *name,
     return NULL;
 
   server = (SilcServerEntry)id_cache->context;
-  
-  if (ret_entry)
-    *ret_entry = id_cache;
 
   if (server && registered &&
       !(server->data.status & SILC_IDLIST_STATUS_REGISTERED))
     return NULL;
 
+  if (ret_entry)
+    *ret_entry = id_cache;
+
   SILC_LOG_DEBUG(("Found"));
 
   return server;
@@ -202,7 +220,7 @@ silc_idlist_find_server_by_conn(SilcIDList id_list, char *hostname,
   SilcIDCacheEntry id_cache = NULL;
   SilcServerEntry server = NULL;
   SilcSocketConnection sock;
- 
+
   SILC_LOG_DEBUG(("Server by hostname %s and port %d", hostname, port));
 
   if (!silc_idcache_get_all(id_list->servers, &list))
@@ -216,10 +234,10 @@ silc_idlist_find_server_by_conn(SilcIDList id_list, char *hostname,
   while (id_cache) {
     server = (SilcServerEntry)id_cache->context;
     sock = (SilcSocketConnection)server->connection;
-    
+
     if (sock && ((sock->hostname && !strcasecmp(sock->hostname, hostname)) ||
 		 (sock->ip && !strcasecmp(sock->ip, hostname)))
-	&& sock->port == port)
+	&& server->id->port == SILC_SWAB_16(port))
       break;
 
     id_cache = NULL;
@@ -228,22 +246,22 @@ silc_idlist_find_server_by_conn(SilcIDList id_list, char *hostname,
     if (!silc_idcache_list_next(list, &id_cache))
       break;
   }
-  
-  silc_idcache_list_free(list);
 
-  if (ret_entry)
-    *ret_entry = id_cache;
+  silc_idcache_list_free(list);
 
   if (server && registered &&
       !(server->data.status & SILC_IDLIST_STATUS_REGISTERED))
     return NULL;
 
+  if (ret_entry)
+    *ret_entry = id_cache;
+
   SILC_LOG_DEBUG(("Found"));
 
   return server;
 }
 
-/* Replaces old Server ID with new one */ 
+/* Replaces old Server ID with new one */
 
 SilcServerEntry
 silc_idlist_replace_server_id(SilcIDList id_list, SilcServerID *old_id,
@@ -251,17 +269,19 @@ silc_idlist_replace_server_id(SilcIDList id_list, SilcServerID *old_id,
 {
   SilcIDCacheEntry id_cache = NULL;
   SilcServerEntry server;
+  char *name;
 
   if (!old_id || !new_id)
     return NULL;
 
   SILC_LOG_DEBUG(("Replacing Server ID"));
 
-  if (!silc_idcache_find_by_id_one(id_list->servers, (void *)old_id, 
+  if (!silc_idcache_find_by_id_one(id_list->servers, (void *)old_id,
 				   &id_cache))
     return NULL;
 
   server = (SilcServerEntry)id_cache->context;
+  name = strdup(id_cache->name);
 
   /* Remove the old entry and add a new one */
 
@@ -270,8 +290,7 @@ silc_idlist_replace_server_id(SilcIDList id_list, SilcServerID *old_id,
   silc_free(server->id);
   server->id = new_id;
 
-  silc_idcache_add(id_list->servers, server->server_name, server->id, 
-		   server, 0, NULL);
+  silc_idcache_add(id_list->servers, name, server->id, server, 0, NULL);
 
   SILC_LOG_DEBUG(("Found"));
 
@@ -282,17 +301,22 @@ silc_idlist_replace_server_id(SilcIDList id_list, SilcServerID *old_id,
 
 int silc_idlist_del_server(SilcIDList id_list, SilcServerEntry entry)
 {
-  SILC_LOG_DEBUG(("Start"));
-
   if (entry) {
     /* Remove from cache */
-    if (entry->id)
-      if (!silc_idcache_del_by_id(id_list->servers, (void *)entry->id))
-	return FALSE;
+    if (!silc_idcache_del_by_context(id_list->servers, entry)) {
+      SILC_LOG_DEBUG(("Unknown server, did not delete"));
+      return FALSE;
+    }
+
+    SILC_LOG_DEBUG(("Deleting server %s id %s", entry->server_name ?
+		    entry->server_name : "",
+		    entry->id ?
+		    silc_id_render(entry->id, SILC_ID_SERVER) : ""));
 
     /* Free data */
     silc_free(entry->server_name);
     silc_free(entry->id);
+    silc_free(entry->server_info);
 
     memset(entry, 'F', sizeof(*entry));
     silc_free(entry);
@@ -318,18 +342,45 @@ int silc_idlist_del_server(SilcIDList id_list, SilcServerEntry entry)
    to be directly connected local client and `router' must be NULL. */
 
 SilcClientEntry
-silc_idlist_add_client(SilcIDList id_list, char *nickname, char *username, 
-		       char *userinfo, SilcClientID *id, 
+silc_idlist_add_client(SilcIDList id_list, char *nickname, char *username,
+		       char *userinfo, SilcClientID *id,
 		       SilcServerEntry router, void *connection,
 		       int expire)
 {
   SilcClientEntry client;
+  char *nicknamec = NULL;
 
   SILC_LOG_DEBUG(("Adding new client entry"));
 
+  /* Normalize name.  This is cached, original is in client context.  */
+  if (nickname) {
+    nicknamec = silc_identifier_check(nickname, strlen(nickname),
+				      SILC_STRING_UTF8, 128, NULL);
+    if (!nicknamec)
+      return NULL;
+  }
+
+  /* Check username. */
+  if (username) {
+    char *u = NULL, *h = NULL;
+    silc_parse_userfqdn(username, &u, &h);
+    if (!u)
+      return NULL;
+    if (!silc_identifier_verify(u, strlen(u), SILC_STRING_UTF8, 128)) {
+      silc_free(u);
+      silc_free(h);
+      return NULL;
+    }
+    if (h && !silc_identifier_verify(h, strlen(h), SILC_STRING_UTF8, 256)) {
+      silc_free(u);
+      silc_free(h);
+      return NULL;
+    }
+  }
+
   client = silc_calloc(1, sizeof(*client));
   client->nickname = nickname;
-  client->username = username;
+  client->username = username ? strdup(username) : NULL;
   client->userinfo = userinfo;
   client->id = id;
   client->router = router;
@@ -337,10 +388,11 @@ silc_idlist_add_client(SilcIDList id_list, char *nickname, char *username,
   client->channels = silc_hash_table_alloc(3, silc_hash_ptr, NULL,
 					   NULL, NULL, NULL, NULL, TRUE);
 
-  if (!silc_idcache_add(id_list->clients, nickname, (void *)client->id, 
+  if (!silc_idcache_add(id_list->clients, nicknamec, (void *)client->id,
 			(void *)client, expire, NULL)) {
     silc_hash_table_free(client->channels);
     silc_free(client);
+    silc_free(nicknamec);
     return NULL;
   }
 
@@ -355,16 +407,20 @@ int silc_idlist_del_client(SilcIDList id_list, SilcClientEntry entry)
   SILC_LOG_DEBUG(("Start"));
 
   if (entry) {
-    /* Remove from cache */
-    if (entry->id)
-      if (!silc_idcache_del_by_context(id_list->clients, entry))
-	return FALSE;
+    /* Remove from cache. Destructor callback deletes stuff. */
+    if (!silc_idcache_del_by_context(id_list->clients, entry)) {
+      SILC_LOG_DEBUG(("Unknown client, did not delete"));
+      return FALSE;
+    }
+
+    assert(!silc_hash_table_count(entry->channels));
 
-    /* Free data */
     silc_free(entry->nickname);
+    silc_free(entry->servername);
     silc_free(entry->username);
     silc_free(entry->userinfo);
     silc_free(entry->id);
+    silc_free(entry->attrs);
     silc_hash_table_free(entry->channels);
 
     memset(entry, 'F', sizeof(*entry));
@@ -376,11 +432,35 @@ int silc_idlist_del_client(SilcIDList id_list, SilcClientEntry entry)
   return FALSE;
 }
 
+/* ID Cache destructor */
+
+void silc_idlist_client_destructor(SilcIDCache cache,
+				   SilcIDCacheEntry entry)
+{
+  SilcClientEntry client;
+
+  client = (SilcClientEntry)entry->context;
+  if (client) {
+    assert(!silc_hash_table_count(client->channels));
+    silc_free(client->nickname);
+    silc_free(client->servername);
+    silc_free(client->username);
+    silc_free(client->userinfo);
+    silc_free(client->id);
+    silc_free(client->attrs);
+    silc_hash_table_free(client->channels);
+
+    memset(client, 'A', sizeof(*client));
+    silc_free(client);
+  }
+}
+
 /* Returns all clients matching requested nickname. Number of clients is
-   returned to `clients_count'. Caller must free the returned table. */
+   returned to `clients_count'. Caller must free the returned table.
+   The 'nickname' must be normalized already. */
 
 int silc_idlist_get_clients_by_nickname(SilcIDList id_list, char *nickname,
-					char *server, 
+					char *server,
 					SilcClientEntry **clients,
 					SilcUInt32 *clients_count)
 {
@@ -392,8 +472,8 @@ int silc_idlist_get_clients_by_nickname(SilcIDList id_list, char *nickname,
   if (!silc_idcache_find_by_name(id_list->clients, nickname, &list))
     return FALSE;
 
-  *clients = silc_realloc(*clients, 
-			  (silc_idcache_list_count(list) + *clients_count) * 
+  *clients = silc_realloc(*clients,
+			  (silc_idcache_list_count(list) + *clients_count) *
 			  sizeof(**clients));
 
   silc_idcache_list_first(list, &id_cache);
@@ -401,16 +481,16 @@ int silc_idlist_get_clients_by_nickname(SilcIDList id_list, char *nickname,
 
   while (silc_idcache_list_next(list, &id_cache))
     (*clients)[(*clients_count)++] = (SilcClientEntry)id_cache->context;
-  
   silc_idcache_list_free(list);
-  
+
   SILC_LOG_DEBUG(("Found total %d clients", *clients_count));
 
   return TRUE;
 }
 
 /* Returns all clients matching requested nickname hash. Number of clients
-   is returned to `clients_count'. Caller must free the returned table. */
+   is returned to `clients_count'. Caller must free the returned table.
+   The 'nickname' must be normalized already. */
 
 int silc_idlist_get_clients_by_hash(SilcIDList id_list, char *nickname,
 				    SilcHash md5hash,
@@ -421,13 +501,10 @@ int silc_idlist_get_clients_by_hash(SilcIDList id_list, char *nickname,
   SilcIDCacheEntry id_cache = NULL;
   unsigned char hash[32];
   SilcClientID client_id;
-  char nick[128 + 1];
 
   SILC_LOG_DEBUG(("Start"));
 
-  memset(nick, 0, sizeof(nick));
-  silc_to_lower(nickname, nick, sizeof(nick) - 1);
-  silc_hash_make(md5hash, nick, strlen(nick), hash);
+  silc_hash_make(md5hash, nickname, strlen(nickname), hash);
 
   /* As the Client ID is hashed in the ID cache by hashing only the hash
      from the Client ID, we can do a lookup with only the hash not the
@@ -438,8 +515,8 @@ int silc_idlist_get_clients_by_hash(SilcIDList id_list, char *nickname,
   if (!silc_idcache_find_by_id(id_list->clients, &client_id, &list))
     return FALSE;
 
-  *clients = silc_realloc(*clients, 
-			  (silc_idcache_list_count(list) + *clients_count) * 
+  *clients = silc_realloc(*clients,
+			  (silc_idcache_list_count(list) + *clients_count) *
 			  sizeof(**clients));
 
   silc_idcache_list_first(list, &id_cache);
@@ -447,9 +524,9 @@ int silc_idlist_get_clients_by_hash(SilcIDList id_list, char *nickname,
 
   while (silc_idcache_list_next(list, &id_cache))
     (*clients)[(*clients_count)++] = (SilcClientEntry)id_cache->context;
-  
+
   silc_idcache_list_free(list);
-  
+
   SILC_LOG_DEBUG(("Found total %d clients", *clients_count));
 
   return TRUE;
@@ -467,28 +544,28 @@ silc_idlist_find_client_by_id(SilcIDList id_list, SilcClientID *id,
   if (!id)
     return NULL;
 
-  SILC_LOG_DEBUG(("Client ID (%s)", 
+  SILC_LOG_DEBUG(("Client ID (%s)",
 		  silc_id_render(id, SILC_ID_CLIENT)));
 
   /* Do extended search since the normal ID comparison function for
      Client ID's compares only the hash from the Client ID and not the
      entire ID. The silc_hash_client_id_compare compares the entire
      Client ID as we want to find one specific Client ID. */
-  if (!silc_idcache_find_by_id_one_ext(id_list->clients, (void *)id, 
-				       NULL, NULL, 
+  if (!silc_idcache_find_by_id_one_ext(id_list->clients, (void *)id,
+				       NULL, NULL,
 				       silc_hash_client_id_compare, NULL,
 				       &id_cache))
     return NULL;
 
   client = (SilcClientEntry)id_cache->context;
 
-  if (ret_entry)
-    *ret_entry = id_cache;
-
   if (client && registered &&
       !(client->data.status & SILC_IDLIST_STATUS_REGISTERED))
     return NULL;
 
+  if (ret_entry)
+    *ret_entry = id_cache;
+
   SILC_LOG_DEBUG(("Found"));
 
   return client;
@@ -503,18 +580,27 @@ silc_idlist_replace_client_id(SilcServer server,
 {
   SilcIDCacheEntry id_cache = NULL;
   SilcClientEntry client;
+  char *nicknamec = NULL;
 
   if (!old_id || !new_id)
     return NULL;
 
   SILC_LOG_DEBUG(("Replacing Client ID"));
 
+  /* Normalize name. This is cached, original is in client context.  */
+  if (nickname) {
+    nicknamec = silc_identifier_check(nickname, strlen(nickname),
+				      SILC_STRING_UTF8, 128, NULL);
+    if (!nicknamec)
+      return NULL;
+  }
+
   /* Do extended search since the normal ID comparison function for
      Client ID's compares only the hash from the Client ID and not the
      entire ID. The silc_hash_client_id_compare compares the entire
      Client ID as we want to find one specific Client ID. */
-  if (!silc_idcache_find_by_id_one_ext(id_list->clients, (void *)old_id, 
-				       NULL, NULL, 
+  if (!silc_idcache_find_by_id_one_ext(id_list->clients, (void *)old_id,
+				       NULL, NULL,
 				       silc_hash_client_id_compare, NULL,
 				       &id_cache))
     return NULL;
@@ -526,7 +612,7 @@ silc_idlist_replace_client_id(SilcServer server,
   if (!silc_idcache_del_by_context(id_list->clients, client))
     return NULL;
 
-  /* Check if anyone is watching this nickname */
+  /* Check if anyone is watching old nickname */
   if (server->server_type == SILC_ROUTER)
     silc_server_check_watcher_list(server, client, nickname,
 				   SILC_NOTIFY_TYPE_NICK_CHANGE);
@@ -536,7 +622,12 @@ silc_idlist_replace_client_id(SilcServer server,
   client->id = new_id;
   client->nickname = nickname ? strdup(nickname) : NULL;
 
-  if (!silc_idcache_add(id_list->clients, client->nickname, client->id, 
+  /* Check if anyone is watching new nickname */
+  if (server->server_type == SILC_ROUTER)
+    silc_server_check_watcher_list(server, client, nickname,
+				   SILC_NOTIFY_TYPE_NICK_CHANGE);
+
+  if (!silc_idcache_add(id_list->clients, nicknamec, client->id,
 			client, 0, NULL))
     return NULL;
 
@@ -545,25 +636,6 @@ silc_idlist_replace_client_id(SilcServer server,
   return client;
 }
 
-/* Client cache entry destructor that is called when the cache is purged. */
-
-void silc_idlist_client_destructor(SilcIDCache cache,
-				   SilcIDCacheEntry entry)
-{
-  SilcClientEntry client;
-
-  client = (SilcClientEntry)entry->context;
-  if (client) {
-    silc_free(client->nickname);
-    silc_free(client->username);
-    silc_free(client->userinfo);
-    silc_free(client->id);
-    silc_hash_table_free(client->channels);
-
-    memset(client, 'F', sizeof(*client));
-    silc_free(client);
-  }
-}
 
 /******************************************************************************
 
@@ -581,8 +653,17 @@ silc_idlist_add_channel(SilcIDList id_list, char *channel_name, int mode,
 			int expire)
 {
   SilcChannelEntry channel;
+  char *channel_namec = NULL;
 
-  SILC_LOG_DEBUG(("Adding new channel entry"));
+  SILC_LOG_DEBUG(("Adding new channel %s", channel_name));
+
+  /* Normalize name.  This is cached, original is in client context.  */
+  if (channel_name) {
+    channel_namec = silc_channel_name_check(channel_name, strlen(channel_name),
+					    SILC_STRING_UTF8, 256, NULL);
+    if (!channel_namec)
+      return NULL;
+  }
 
   channel = silc_calloc(1, sizeof(*channel));
   channel->channel_name = channel_name;
@@ -591,7 +672,7 @@ silc_idlist_add_channel(SilcIDList id_list, char *channel_name, int mode,
   channel->router = router;
   channel->channel_key = channel_key;
   channel->hmac = hmac;
-  channel->created = time(0);
+  channel->created = channel->updated = time(0);
   if (!channel->hmac)
     if (!silc_hmac_alloc(SILC_DEFAULT_HMAC, NULL, &channel->hmac)) {
       silc_free(channel);
@@ -601,11 +682,12 @@ silc_idlist_add_channel(SilcIDList id_list, char *channel_name, int mode,
   channel->user_list = silc_hash_table_alloc(3, silc_hash_ptr, NULL, NULL,
 					     NULL, NULL, NULL, TRUE);
 
-  if (!silc_idcache_add(id_list->channels, channel->channel_name, 
+  if (!silc_idcache_add(id_list->channels, channel_namec,
 			(void *)channel->id, (void *)channel, expire, NULL)) {
     silc_hmac_free(channel->hmac);
     silc_hash_table_free(channel->user_list);
     silc_free(channel);
+    silc_free(channel_namec);
     return NULL;
   }
 
@@ -621,7 +703,8 @@ static void silc_idlist_del_channel_foreach(void *key, void *context,
   SilcChannelClientEntry chl = (SilcChannelClientEntry)context;
 
   SILC_LOG_DEBUG(("Removing client %s from channel %s",
-		  chl->client->nickname, chl->channel->channel_name));
+		  chl->client->nickname ? chl->client->nickname :
+		  (unsigned char *)"", chl->channel->channel_name));
 
   /* Remove the context from the client's channel hash table as that
      table and channel's user_list hash table share this same context. */
@@ -633,16 +716,17 @@ static void silc_idlist_del_channel_foreach(void *key, void *context,
 
 int silc_idlist_del_channel(SilcIDList id_list, SilcChannelEntry entry)
 {
-  SILC_LOG_DEBUG(("Start"));
-
   if (entry) {
     /* Remove from cache */
-    if (entry->id)
-      if (!silc_idcache_del_by_id(id_list->channels, (void *)entry->id))
-	return FALSE;
+    if (!silc_idcache_del_by_context(id_list->channels, entry)) {
+      SILC_LOG_DEBUG(("Unknown channel, did not delete"));
+      return FALSE;
+    }
+
+    SILC_LOG_DEBUG(("Deleting channel %s", entry->channel_name));
 
     /* Free all client entrys from the users list. The silc_hash_table_free
-       will free all the entries so they are not freed at the foreach 
+       will free all the entries so they are not freed at the foreach
        callback. */
     silc_hash_table_foreach(entry->user_list, silc_idlist_del_channel_foreach,
 			    NULL);
@@ -652,6 +736,12 @@ int silc_idlist_del_channel(SilcIDList id_list, SilcChannelEntry entry)
     silc_free(entry->channel_name);
     silc_free(entry->id);
     silc_free(entry->topic);
+
+    if (entry->invite_list)
+      silc_hash_table_free(entry->invite_list);
+    if (entry->ban_list)
+      silc_hash_table_free(entry->ban_list);
+
     if (entry->channel_key)
       silc_cipher_free(entry->channel_key);
     if (entry->key) {
@@ -665,6 +755,8 @@ int silc_idlist_del_channel(SilcIDList id_list, SilcChannelEntry entry)
     silc_free(entry->rekey);
     if (entry->founder_key)
       silc_pkcs_public_key_free(entry->founder_key);
+    if (entry->channel_pubkeys)
+      silc_hash_table_free(entry->channel_pubkeys);
 
     memset(entry, 'F', sizeof(*entry));
     silc_free(entry);
@@ -675,7 +767,7 @@ int silc_idlist_del_channel(SilcIDList id_list, SilcChannelEntry entry)
 }
 
 /* Finds channel by channel name. Channel names are unique and they
-   are not case-sensitive. */
+   are not case-sensitive.  The 'name' must be normalized already. */
 
 SilcChannelEntry
 silc_idlist_find_channel_by_name(SilcIDList id_list, char *name,
@@ -693,6 +785,9 @@ silc_idlist_find_channel_by_name(SilcIDList id_list, char *name,
 
   SILC_LOG_DEBUG(("Found"));
 
+  /* Touch channel */
+  ((SilcChannelEntry)id_cache->context)->updated = time(NULL);
+
   return id_cache->context;
 }
 
@@ -721,6 +816,9 @@ silc_idlist_find_channel_by_id(SilcIDList id_list, SilcChannelID *id,
 
   SILC_LOG_DEBUG(("Found"));
 
+  /* Touch channel */
+  channel->updated = time(NULL);
+
   return channel;
 }
 
@@ -733,17 +831,19 @@ silc_idlist_replace_channel_id(SilcIDList id_list, SilcChannelID *old_id,
 {
   SilcIDCacheEntry id_cache = NULL;
   SilcChannelEntry channel;
+  char *name;
 
   if (!old_id || !new_id)
     return NULL;
 
   SILC_LOG_DEBUG(("Replacing Channel ID"));
 
-  if (!silc_idcache_find_by_id_one(id_list->channels, (void *)old_id, 
+  if (!silc_idcache_find_by_id_one(id_list->channels, (void *)old_id,
 				   &id_cache))
     return NULL;
 
   channel = (SilcChannelEntry)id_cache->context;
+  name = strdup(id_cache->name);
 
   /* Remove the old entry and add a new one */
 
@@ -752,11 +852,13 @@ silc_idlist_replace_channel_id(SilcIDList id_list, SilcChannelID *old_id,
   silc_free(channel->id);
   channel->id = new_id;
 
-  silc_idcache_add(id_list->channels, channel->channel_name, channel->id, 
-		   channel, 0, NULL);
+  silc_idcache_add(id_list->channels, name, channel->id, channel, 0, NULL);
 
   SILC_LOG_DEBUG(("Replaced"));
 
+  /* Touch channel */
+  channel->updated = time(NULL);
+
   return channel;
 }
 
@@ -779,14 +881,14 @@ silc_idlist_get_channels(SilcIDList id_list, SilcChannelID *channel_id,
       return NULL;
 
     channels = silc_calloc(silc_idcache_list_count(list), sizeof(*channels));
-    
+
     i = 0;
     silc_idcache_list_first(list, &id_cache);
     channels[i++] = (SilcChannelEntry)id_cache->context;
-    
+
     while (silc_idcache_list_next(list, &id_cache))
       channels[i++] = (SilcChannelEntry)id_cache->context;
-    
+
     silc_idcache_list_free(list);
   } else {
     if (!silc_idcache_find_by_id_one(id_list->channels, channel_id, &id_cache))
