@@ -4,7 +4,7 @@
 
   Author: Pekka Riikonen <priikone@silcnet.org>
 
-  Copyright (C) 2002 - 2003 Pekka Riikonen
+  Copyright (C) 2002 - 2005 Pekka Riikonen
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: server_query.c,v 1.19.2.10 2003/12/20 13:06:10 priikone Exp $ */
+/* $Id: server_query.c,v 1.19.2.17 2005/04/10 09:04:14 priikone Exp $ */
 
 #include "serverincludes.h"
 #include "server_internal.h"
@@ -43,7 +43,7 @@ typedef struct {
   void *id;			    /* ID */
   SilcIdType id_type;		    /* ID type */
   unsigned int index : 15;	    /* Index to IDs */
-  unsigned int type : 2;   	    /* 0 = take from query->ids, 0 = take
+  unsigned int type : 2;   	    /* 0 = take from query->ids, 1 = take
 				       from args, 2 = no args in error. */
   unsigned int error : 7;	    /* The actual error (SilcStatus) */
 } *SilcServerQueryError;
@@ -51,10 +51,10 @@ typedef struct {
 /* Query session context */
 typedef struct {
   /* Queried data */
-  char *nickname;		    /* Queried nickname */
+  char *nickname;		    /* Queried nickname, normalized */
   char *nick_server;		    /* Queried nickname's server */
-  char *server_name;		    /* Queried server name */
-  char *channel_name;		    /* Queried channel name */
+  char *server_name;		    /* Queried server name, normalized */
+  char *channel_name;		    /* Queried channel name, normalized */
   SilcServerQueryID ids;	    /* Queried IDs */
   SilcUInt32 ids_count;		    /* number of queried IDs */
   SilcUInt32 reply_count;	    /* Requested reply count */
@@ -323,6 +323,13 @@ void silc_server_query_send_router_reply(void *context, void *reply)
 
   SILC_LOG_DEBUG(("Received reply from router to query"));
 
+  /* If the original command caller has gone away, just stop. */
+  if (query->cmd->sock->users == 1) {
+    SILC_LOG_DEBUG(("Original command caller vanished"));
+    silc_server_query_free(query);
+    return;
+  }
+
   /* Check if router sent error reply */
   if (cmdr && !silc_command_get_status(cmdr->payload, NULL, NULL)) {
     SilcBuffer buffer;
@@ -399,6 +406,18 @@ void silc_server_query_parse(SilcServer server, SilcServerQuery query)
 	return;
       }
 
+      /* Check nickname */
+      tmp = silc_identifier_check(query->nickname, strlen(query->nickname),
+				  SILC_STRING_UTF8, 128, &tmp_len);
+      if (!tmp) {
+	silc_server_query_send_error(server, query,
+				     SILC_STATUS_ERR_BAD_NICKNAME, 0);
+	silc_server_query_free(query);
+	return;
+      }
+      silc_free(query->nickname);
+      query->nickname = tmp;
+
     } else {
       /* Parse the IDs included in the query */
       query->ids = silc_calloc(argc, sizeof(*query->ids));
@@ -466,6 +485,18 @@ void silc_server_query_parse(SilcServer server, SilcServerQuery query)
       return;
     }
 
+    /* Check nickname */
+    tmp = silc_identifier_check(query->nickname, strlen(query->nickname),
+				SILC_STRING_UTF8, 128, &tmp_len);
+    if (!tmp) {
+      silc_server_query_send_error(server, query,
+				   SILC_STATUS_ERR_BAD_NICKNAME, 0);
+      silc_server_query_free(query);
+      return;
+    }
+    silc_free(query->nickname);
+    query->nickname = tmp;
+
     /* Get the max count of reply messages allowed */
     tmp = silc_argument_get_arg_type(cmd->args, 2, &tmp_len);
     if (tmp && tmp_len == sizeof(SilcUInt32))
@@ -485,17 +516,49 @@ void silc_server_query_parse(SilcServer server, SilcServerQuery query)
 	    !silc_parse_userfqdn(tmp, &query->nickname, &query->nick_server))
 	  silc_server_query_add_error(server, query, 1, 1,
 				      SILC_STATUS_ERR_BAD_NICKNAME);
+
+	/* Check nickname */
+	tmp = silc_identifier_check(query->nickname, strlen(query->nickname),
+				    SILC_STRING_UTF8, 128, &tmp_len);
+	if (!tmp) {
+	  silc_server_query_send_error(server, query,
+				       SILC_STATUS_ERR_BAD_NICKNAME, 0);
+	  silc_server_query_free(query);
+	  return;
+	}
+	silc_free(query->nickname);
+	query->nickname = tmp;
       }
 
       /* Try get server name */
       tmp = silc_argument_get_arg_type(cmd->args, 2, &tmp_len);
-      if (tmp)
-	query->server_name = silc_memdup(tmp, tmp_len);
+      if (tmp) {
+	/* Check server name */
+	tmp = silc_identifier_check(tmp, tmp_len, SILC_STRING_UTF8,
+				    256, &tmp_len);
+	if (!tmp) {
+	  silc_server_query_send_error(server, query,
+				       SILC_STATUS_ERR_BAD_SERVER, 0);
+	  silc_server_query_free(query);
+	  return;
+	}
+	query->server_name = tmp;
+      }
 
       /* Get channel name */
       tmp = silc_argument_get_arg_type(cmd->args, 3, &tmp_len);
-      if (tmp && tmp_len <= 256)
-	query->channel_name = silc_memdup(tmp, tmp_len);
+      if (tmp && tmp_len <= 256) {
+	/* Check channel name */
+	tmp = silc_identifier_check(tmp, tmp_len, SILC_STRING_UTF8,
+				    256, &tmp_len);
+	if (!tmp) {
+	  silc_server_query_send_error(server, query,
+				       SILC_STATUS_ERR_BAD_CHANNEL, 0);
+	  silc_server_query_free(query);
+	  return;
+	}
+	query->channel_name = tmp;
+      }
 
       if (!query->nickname && !query->server_name && !query->channel_name) {
 	silc_server_query_send_error(server, query,
@@ -613,7 +676,7 @@ void silc_server_query_check_attributes(SilcServer server,
 
   /* If no clients were found, we only check the attributes
      if the user wasn't searching for nickname/ids */
-  if (!*clients) {
+  if (!(*clients)) {
     no_clients = TRUE;
     if (query->nickname || query->ids_count)
       return;
@@ -1225,6 +1288,13 @@ void silc_server_query_resolve_reply(void *context, void *reply)
 
   SILC_LOG_DEBUG(("Reprocess the query"));
 
+  /* If the original command caller has gone away, just stop. */
+  if (query->cmd->sock->users == 1) {
+    SILC_LOG_DEBUG(("Original command caller vanished"));
+    silc_server_query_free(query);
+    return;
+  }
+
   /* We have received all queries.  Now re-search all information required
      to complete this query.  Reason we cannot save the values found in
      the first search is that SilcClientEntry, SilcServerEntry and
@@ -1258,7 +1328,7 @@ void silc_server_query_send_reply(SilcServer server,
   SilcUInt32 len;
   SilcBuffer idp;
   int i, k, valid_count;
-  char nh[256], uh[256];
+  char nh[384], uh[384];
   bool sent_reply = FALSE;
 
   SILC_LOG_DEBUG(("Sending reply to query"));
@@ -1346,7 +1416,6 @@ void silc_server_query_send_reply(SilcServer server,
 		       "      : "), entry->nickname));
 
       idp = silc_id_payload_encode(entry->id, SILC_ID_CLIENT);
-      memset(uh, 0, sizeof(uh));
       memset(nh, 0, sizeof(nh));
 
       silc_strncat(nh, sizeof(nh), entry->nickname, strlen(entry->nickname));
@@ -1374,6 +1443,8 @@ void silc_server_query_send_reply(SilcServer server,
 
 	  memset(fempty, 0, sizeof(fempty));
 	  memset(idle, 0, sizeof(idle));
+	  memset(uh, 0, sizeof(uh));
+
 	  silc_strncat(uh, sizeof(uh), entry->username,
 		       strlen(entry->username));
 	  if (!strchr(entry->username, '@') && entry->connection) {
@@ -1482,7 +1553,7 @@ void silc_server_query_send_reply(SilcServer server,
       case SILC_COMMAND_WHOWAS:
 	silc_strncat(uh, sizeof(uh), entry->username, strlen(entry->username));
 	if (!strchr(entry->username, '@'))
-	  silc_strncat(uh, sizeof(uh), "@*private*", 10);
+	  silc_strncat(uh, sizeof(uh), "@-private-", 10);
 
 	/* Send command reply */
 	silc_server_send_command_reply(server, cmd->sock, query->querycmd,
@@ -1508,9 +1579,14 @@ void silc_server_query_send_reply(SilcServer server,
       /* Not one valid entry was found, send error.  If nickname was used
 	 in query send error based on that, otherwise the query->errors
 	 already includes proper errors. */
-      if (query->nickname || (!query->nickname && !query->ids && query->attrs))
+      if (query->nickname || (!query->ids && query->attrs))
 	silc_server_query_add_error(server, query, 1, 1,
 				    SILC_STATUS_ERR_NO_SUCH_NICK);
+
+      /* Make sure some error is sent */
+      if (!query->errors_count && !servers_count && !channels_count)
+	silc_server_query_add_error(server, query, 2, 0,
+				    SILC_STATUS_ERR_NOT_ENOUGH_PARAMS);
     }
   }
 
